@@ -36,8 +36,7 @@
 #ifdef USE_MMX
 
 #include <mmintrin.h>
-
-#include "pixman-mmx.h"
+#include "pixman-private.h"
 
 #define noVERBOSE
 
@@ -437,7 +436,8 @@ combine (const uint32_t *src, const uint32_t *mask)
 }
 
 static FASTCALL void
-mmxCombineOverU (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
+mmxCombineOverU (pixman_implementation_t *imp, pixman_op_t op,
+		 uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = dest + width;
 
@@ -446,7 +446,7 @@ mmxCombineOverU (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int
 	uint32_t a = ssrc >> 24;
 	if (a == 0xff) {
 	    *dest = ssrc;
-	} else if (a) {
+	} else if (ssrc) {
 	    __m64 s, sa;
 	    s = load8888(ssrc);
 	    sa = expand_alpha(s);
@@ -461,7 +461,8 @@ mmxCombineOverU (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int
 }
 
 static FASTCALL void
-mmxCombineOverReverseU (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
+mmxCombineOverReverseU (pixman_implementation_t *imp, pixman_op_t op,
+			uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = dest + width;
 
@@ -480,7 +481,8 @@ mmxCombineOverReverseU (uint32_t *dest, const uint32_t *src, const uint32_t *mas
 }
 
 static FASTCALL void
-mmxCombineInU (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
+mmxCombineInU (pixman_implementation_t *imp, pixman_op_t op,
+	       uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = dest + width;
 
@@ -500,7 +502,8 @@ mmxCombineInU (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int wi
 }
 
 static FASTCALL void
-mmxCombineInReverseU (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
+mmxCombineInReverseU (pixman_implementation_t *imp, pixman_op_t op,
+		      uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = dest + width;
 
@@ -520,7 +523,8 @@ mmxCombineInReverseU (uint32_t *dest, const uint32_t *src, const uint32_t *mask,
 }
 
 static FASTCALL void
-mmxCombineOutU (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
+mmxCombineOutU (pixman_implementation_t *imp, pixman_op_t op,
+		uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = dest + width;
 
@@ -541,7 +545,8 @@ mmxCombineOutU (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int w
 }
 
 static FASTCALL void
-mmxCombineOutReverseU (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
+mmxCombineOutReverseU (pixman_implementation_t *imp, pixman_op_t op,
+		       uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = dest + width;
 
@@ -562,7 +567,8 @@ mmxCombineOutReverseU (uint32_t *dest, const uint32_t *src, const uint32_t *mask
 }
 
 static FASTCALL void
-mmxCombineAtopU (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
+mmxCombineAtopU (pixman_implementation_t *imp, pixman_op_t op,
+		 uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = dest + width;
 
@@ -584,7 +590,8 @@ mmxCombineAtopU (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int
 }
 
 static FASTCALL void
-mmxCombineAtopReverseU (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
+mmxCombineAtopReverseU (pixman_implementation_t *imp, pixman_op_t op,
+			uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end;
 
@@ -608,7 +615,8 @@ mmxCombineAtopReverseU (uint32_t *dest, const uint32_t *src, const uint32_t *mas
 }
 
 static FASTCALL void
-mmxCombineXorU (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
+mmxCombineXorU (pixman_implementation_t *imp, pixman_op_t op,
+		uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = dest + width;
 
@@ -631,7 +639,8 @@ mmxCombineXorU (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int w
 }
 
 static FASTCALL void
-mmxCombineAddU (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
+mmxCombineAddU (pixman_implementation_t *imp, pixman_op_t op,
+		uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = dest + width;
     while (dest < end) {
@@ -649,7 +658,8 @@ mmxCombineAddU (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int w
 }
 
 static FASTCALL void
-mmxCombineSaturateU (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
+mmxCombineSaturateU (pixman_implementation_t *imp, pixman_op_t op,
+		     uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = dest + width;
     while (dest < end) {
@@ -677,7 +687,8 @@ mmxCombineSaturateU (uint32_t *dest, const uint32_t *src, const uint32_t *mask,
 
 
 static FASTCALL void
-mmxCombineSrcC (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
+mmxCombineSrcC (pixman_implementation_t *imp, pixman_op_t op,
+		uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = src + width;
     while (src < end) {
@@ -693,7 +704,8 @@ mmxCombineSrcC (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int w
 }
 
 static FASTCALL void
-mmxCombineOverC (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
+mmxCombineOverC (pixman_implementation_t *imp, pixman_op_t op,
+		 uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = src + width;
     while (src < end) {
@@ -712,7 +724,8 @@ mmxCombineOverC (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int
 }
 
 static FASTCALL void
-mmxCombineOverReverseC (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
+mmxCombineOverReverseC (pixman_implementation_t *imp, pixman_op_t op,
+			uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = src + width;
     while (src < end) {
@@ -732,7 +745,8 @@ mmxCombineOverReverseC (uint32_t *dest, const uint32_t *src, const uint32_t *mas
 
 
 static FASTCALL void
-mmxCombineInC (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
+mmxCombineInC (pixman_implementation_t *imp, pixman_op_t op,
+	       uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = src + width;
     while (src < end) {
@@ -751,7 +765,8 @@ mmxCombineInC (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int wi
 }
 
 static FASTCALL void
-mmxCombineInReverseC (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
+mmxCombineInReverseC (pixman_implementation_t *imp, pixman_op_t op,
+		      uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = src + width;
     while (src < end) {
@@ -770,7 +785,8 @@ mmxCombineInReverseC (uint32_t *dest, const uint32_t *src, const uint32_t *mask,
 }
 
 static FASTCALL void
-mmxCombineOutC (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
+mmxCombineOutC (pixman_implementation_t *imp, pixman_op_t op,
+		uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = src + width;
     while (src < end) {
@@ -790,7 +806,8 @@ mmxCombineOutC (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int w
 }
 
 static FASTCALL void
-mmxCombineOutReverseC (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
+mmxCombineOutReverseC (pixman_implementation_t *imp, pixman_op_t op,
+		       uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = src + width;
     while (src < end) {
@@ -810,7 +827,8 @@ mmxCombineOutReverseC (uint32_t *dest, const uint32_t *src, const uint32_t *mask
 }
 
 static FASTCALL void
-mmxCombineAtopC (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
+mmxCombineAtopC (pixman_implementation_t *imp, pixman_op_t op,
+		 uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = src + width;
     while (src < end) {
@@ -832,7 +850,8 @@ mmxCombineAtopC (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int
 }
 
 static FASTCALL void
-mmxCombineAtopReverseC (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
+mmxCombineAtopReverseC (pixman_implementation_t *imp, pixman_op_t op,
+			uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = src + width;
     while (src < end) {
@@ -854,7 +873,8 @@ mmxCombineAtopReverseC (uint32_t *dest, const uint32_t *src, const uint32_t *mas
 }
 
 static FASTCALL void
-mmxCombineXorC (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
+mmxCombineXorC (pixman_implementation_t *imp, pixman_op_t op,
+		uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = src + width;
     while (src < end) {
@@ -877,7 +897,8 @@ mmxCombineXorC (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int w
 }
 
 static FASTCALL void
-mmxCombineAddC (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
+mmxCombineAddC (pixman_implementation_t *imp, pixman_op_t op,
+		uint32_t *dest, const uint32_t *src, const uint32_t *mask, int width)
 {
     const uint32_t *end = src + width;
     while (src < end) {
@@ -894,61 +915,22 @@ mmxCombineAddC (uint32_t *dest, const uint32_t *src, const uint32_t *mask, int w
     _mm_empty();
 }
 
-void
-fbComposeSetupMMX(void)
-{
-    static pixman_bool_t initialized = FALSE;
-
-    if (initialized)
-	return;
-    
-    /* check if we have MMX support and initialize accordingly */
-    if (pixman_have_mmx())
-    {
-        pixman_composeFunctions.combineU[PIXMAN_OP_OVER] = mmxCombineOverU;
-        pixman_composeFunctions.combineU[PIXMAN_OP_OVER_REVERSE] = mmxCombineOverReverseU;
-        pixman_composeFunctions.combineU[PIXMAN_OP_IN] = mmxCombineInU;
-        pixman_composeFunctions.combineU[PIXMAN_OP_IN_REVERSE] = mmxCombineInReverseU;
-        pixman_composeFunctions.combineU[PIXMAN_OP_OUT] = mmxCombineOutU;
-        pixman_composeFunctions.combineU[PIXMAN_OP_OUT_REVERSE] = mmxCombineOutReverseU;
-        pixman_composeFunctions.combineU[PIXMAN_OP_ATOP] = mmxCombineAtopU;
-        pixman_composeFunctions.combineU[PIXMAN_OP_ATOP_REVERSE] = mmxCombineAtopReverseU;
-        pixman_composeFunctions.combineU[PIXMAN_OP_XOR] = mmxCombineXorU; 
-	pixman_composeFunctions.combineU[PIXMAN_OP_ADD] = mmxCombineAddU;
-        pixman_composeFunctions.combineU[PIXMAN_OP_SATURATE] = mmxCombineSaturateU;
-
-        pixman_composeFunctions.combineC[PIXMAN_OP_SRC] = mmxCombineSrcC;
-        pixman_composeFunctions.combineC[PIXMAN_OP_OVER] = mmxCombineOverC;
-        pixman_composeFunctions.combineC[PIXMAN_OP_OVER_REVERSE] = mmxCombineOverReverseC;
-        pixman_composeFunctions.combineC[PIXMAN_OP_IN] = mmxCombineInC;
-        pixman_composeFunctions.combineC[PIXMAN_OP_IN_REVERSE] = mmxCombineInReverseC;
-        pixman_composeFunctions.combineC[PIXMAN_OP_OUT] = mmxCombineOutC;
-        pixman_composeFunctions.combineC[PIXMAN_OP_OUT_REVERSE] = mmxCombineOutReverseC;
-        pixman_composeFunctions.combineC[PIXMAN_OP_ATOP] = mmxCombineAtopC;
-        pixman_composeFunctions.combineC[PIXMAN_OP_ATOP_REVERSE] = mmxCombineAtopReverseC;
-        pixman_composeFunctions.combineC[PIXMAN_OP_XOR] = mmxCombineXorC;
-        pixman_composeFunctions.combineC[PIXMAN_OP_ADD] = mmxCombineAddC;
-    }
-
-    initialized = TRUE;
-}
-
-
 /* ------------------ MMX code paths called from fbpict.c ----------------------- */
 
 static void
-fbCompositeSolid_nx8888mmx (pixman_op_t op,
+fbCompositeSolid_nx8888mmx (pixman_implementation_t *imp,
+			    pixman_op_t op,
 			    pixman_image_t * pSrc,
 			    pixman_image_t * pMask,
 			    pixman_image_t * pDst,
-			    int16_t	xSrc,
-			    int16_t	ySrc,
-			    int16_t	xMask,
-			    int16_t	yMask,
-			    int16_t	xDst,
-			    int16_t	yDst,
-			    uint16_t	width,
-			    uint16_t	height)
+			    int32_t	xSrc,
+			    int32_t	ySrc,
+			    int32_t	xMask,
+			    int32_t	yMask,
+			    int32_t	xDst,
+			    int32_t	yDst,
+			    int32_t	width,
+			    int32_t	height)
 {
     uint32_t	src;
     uint32_t	*dstLine, *dst;
@@ -1015,18 +997,19 @@ fbCompositeSolid_nx8888mmx (pixman_op_t op,
 }
 
 static void
-fbCompositeSolid_nx0565mmx (pixman_op_t op,
+fbCompositeSolid_nx0565mmx (pixman_implementation_t *imp,
+			    pixman_op_t op,
 			    pixman_image_t * pSrc,
 			    pixman_image_t * pMask,
 			    pixman_image_t * pDst,
-			    int16_t	xSrc,
-			    int16_t	ySrc,
-			    int16_t	xMask,
-			    int16_t	yMask,
-			    int16_t	xDst,
-			    int16_t	yDst,
-			    uint16_t	width,
-			    uint16_t	height)
+			    int32_t	xSrc,
+			    int32_t	ySrc,
+			    int32_t	xMask,
+			    int32_t	yMask,
+			    int32_t	xDst,
+			    int32_t	yDst,
+			    int32_t	width,
+			    int32_t	height)
 {
     uint32_t	src;
     uint16_t	*dstLine, *dst;
@@ -1100,18 +1083,19 @@ fbCompositeSolid_nx0565mmx (pixman_op_t op,
 }
 
 static void
-fbCompositeSolidMask_nx8888x8888Cmmx (pixman_op_t op,
+fbCompositeSolidMask_nx8888x8888Cmmx (pixman_implementation_t *imp,
+				      pixman_op_t op,
 				      pixman_image_t * pSrc,
 				      pixman_image_t * pMask,
 				      pixman_image_t * pDst,
-				      int16_t	xSrc,
-				      int16_t	ySrc,
-				      int16_t	xMask,
-				      int16_t	yMask,
-				      int16_t	xDst,
-				      int16_t	yDst,
-				      uint16_t	width,
-				      uint16_t	height)
+				      int32_t	xSrc,
+				      int32_t	ySrc,
+				      int32_t	xMask,
+				      int32_t	yMask,
+				      int32_t	xDst,
+				      int32_t	yDst,
+				      int32_t	width,
+				      int32_t	height)
 {
     uint32_t	src, srca;
     uint32_t	*dstLine;
@@ -1203,18 +1187,19 @@ fbCompositeSolidMask_nx8888x8888Cmmx (pixman_op_t op,
 }
 
 static void
-fbCompositeSrc_8888x8x8888mmx (pixman_op_t op,
+fbCompositeSrc_8888x8x8888mmx (pixman_implementation_t *imp,
+			       pixman_op_t op,
 			       pixman_image_t * pSrc,
 			       pixman_image_t * pMask,
 			       pixman_image_t * pDst,
-			       int16_t	xSrc,
-			       int16_t	ySrc,
-			       int16_t      xMask,
-			       int16_t      yMask,
-			       int16_t      xDst,
-			       int16_t      yDst,
-			       uint16_t     width,
-			       uint16_t     height)
+			       int32_t	xSrc,
+			       int32_t	ySrc,
+			       int32_t      xMask,
+			       int32_t      yMask,
+			       int32_t      xDst,
+			       int32_t      yDst,
+			       int32_t     width,
+			       int32_t     height)
 {
     uint32_t	*dstLine, *dst;
     uint32_t	*srcLine, *src;
@@ -1287,18 +1272,19 @@ fbCompositeSrc_8888x8x8888mmx (pixman_op_t op,
 }
 
 static void
-fbCompositeSrc_x888xnx8888mmx (pixman_op_t op,
+fbCompositeSrc_x888xnx8888mmx (pixman_implementation_t *imp,
+			       pixman_op_t op,
 			       pixman_image_t * pSrc,
 			       pixman_image_t * pMask,
 			       pixman_image_t * pDst,
-			       int16_t	xSrc,
-			       int16_t	ySrc,
-			       int16_t      xMask,
-			       int16_t      yMask,
-			       int16_t      xDst,
-			       int16_t      yDst,
-			       uint16_t     width,
-			       uint16_t     height)
+			       int32_t	xSrc,
+			       int32_t	ySrc,
+			       int32_t      xMask,
+			       int32_t      yMask,
+			       int32_t      xDst,
+			       int32_t      yDst,
+			       int32_t     width,
+			       int32_t     height)
 {
     uint32_t	*dstLine, *dst;
     uint32_t	*srcLine, *src;
@@ -1421,18 +1407,19 @@ fbCompositeSrc_x888xnx8888mmx (pixman_op_t op,
 }
 
 static void
-fbCompositeSrc_8888x8888mmx (pixman_op_t op,
+fbCompositeSrc_8888x8888mmx (pixman_implementation_t *imp,
+			     pixman_op_t op,
 			     pixman_image_t * pSrc,
 			     pixman_image_t * pMask,
 			     pixman_image_t * pDst,
-			     int16_t	xSrc,
-			     int16_t	ySrc,
-			     int16_t      xMask,
-			     int16_t      yMask,
-			     int16_t      xDst,
-			     int16_t      yDst,
-			     uint16_t     width,
-			     uint16_t     height)
+			     int32_t	xSrc,
+			     int32_t	ySrc,
+			     int32_t      xMask,
+			     int32_t      yMask,
+			     int32_t      xDst,
+			     int32_t      yDst,
+			     int32_t     width,
+			     int32_t     height)
 {
     uint32_t	*dstLine, *dst;
     uint32_t	*srcLine, *src;
@@ -1460,7 +1447,7 @@ fbCompositeSrc_8888x8888mmx (pixman_op_t op,
 	    a = s >> 24;
 	    if (a == 0xff)
 		*dst = s;
-	    else if (a) {
+	    else if (s) {
 		__m64 ms, sa;
 		ms = load8888(s);
 		sa = expand_alpha(ms);
@@ -1473,18 +1460,19 @@ fbCompositeSrc_8888x8888mmx (pixman_op_t op,
 }
 
 static void
-fbCompositeSrc_8888x0565mmx (pixman_op_t op,
+fbCompositeSrc_8888x0565mmx (pixman_implementation_t *imp,
+			     pixman_op_t op,
 			     pixman_image_t * pSrc,
 			     pixman_image_t * pMask,
 			     pixman_image_t * pDst,
-			     int16_t      xSrc,
-			     int16_t      ySrc,
-			     int16_t      xMask,
-			     int16_t      yMask,
-			     int16_t      xDst,
-			     int16_t      yDst,
-			     uint16_t     width,
-			     uint16_t     height)
+			     int32_t      xSrc,
+			     int32_t      ySrc,
+			     int32_t      xMask,
+			     int32_t      yMask,
+			     int32_t      xDst,
+			     int32_t      yDst,
+			     int32_t     width,
+			     int32_t     height)
 {
     uint16_t	*dstLine, *dst;
     uint32_t	*srcLine, *src;
@@ -1574,18 +1562,19 @@ fbCompositeSrc_8888x0565mmx (pixman_op_t op,
 }
 
 static void
-fbCompositeSolidMask_nx8x8888mmx (pixman_op_t op,
+fbCompositeSolidMask_nx8x8888mmx (pixman_implementation_t *imp,
+				  pixman_op_t op,
 				  pixman_image_t * pSrc,
 				  pixman_image_t * pMask,
 				  pixman_image_t * pDst,
-				  int16_t      xSrc,
-				  int16_t      ySrc,
-				  int16_t      xMask,
-				  int16_t      yMask,
-				  int16_t      xDst,
-				  int16_t      yDst,
-				  uint16_t     width,
-				  uint16_t     height)
+				  int32_t      xSrc,
+				  int32_t      ySrc,
+				  int32_t      xMask,
+				  int32_t      yMask,
+				  int32_t      xDst,
+				  int32_t      yDst,
+				  int32_t     width,
+				  int32_t     height)
 {
     uint32_t	src, srca;
     uint32_t	*dstLine, *dst;
@@ -1845,18 +1834,19 @@ pixman_fill_mmx (uint32_t *bits,
 }
 
 static void
-fbCompositeSolidMaskSrc_nx8x8888mmx (pixman_op_t op,
+fbCompositeSolidMaskSrc_nx8x8888mmx (pixman_implementation_t *imp,
+				     pixman_op_t op,
 				     pixman_image_t * pSrc,
 				     pixman_image_t * pMask,
 				     pixman_image_t * pDst,
-				     int16_t      xSrc,
-				     int16_t      ySrc,
-				     int16_t      xMask,
-				     int16_t      yMask,
-				     int16_t      xDst,
-				     int16_t      yDst,
-				     uint16_t     width,
-				     uint16_t     height)
+				     int32_t      xSrc,
+				     int32_t      ySrc,
+				     int32_t      xMask,
+				     int32_t      yMask,
+				     int32_t      xDst,
+				     int32_t      yDst,
+				     int32_t     width,
+				     int32_t     height)
 {
     uint32_t	src, srca;
     uint32_t	*dstLine, *dst;
@@ -1976,18 +1966,19 @@ fbCompositeSolidMaskSrc_nx8x8888mmx (pixman_op_t op,
 }
 
 static void
-fbCompositeSolidMask_nx8x0565mmx (pixman_op_t op,
+fbCompositeSolidMask_nx8x0565mmx (pixman_implementation_t *imp,
+				  pixman_op_t op,
 				  pixman_image_t * pSrc,
 				  pixman_image_t * pMask,
 				  pixman_image_t * pDst,
-				  int16_t      xSrc,
-				  int16_t      ySrc,
-				  int16_t      xMask,
-				  int16_t      yMask,
-				  int16_t      xDst,
-				  int16_t      yDst,
-				  uint16_t     width,
-				  uint16_t     height)
+				  int32_t      xSrc,
+				  int32_t      ySrc,
+				  int32_t      xMask,
+				  int32_t      yMask,
+				  int32_t      xDst,
+				  int32_t      yDst,
+				  int32_t     width,
+				  int32_t     height)
 {
     uint32_t	src, srca;
     uint16_t	*dstLine, *dst;
@@ -2108,18 +2099,19 @@ fbCompositeSolidMask_nx8x0565mmx (pixman_op_t op,
 }
 
 static void
-fbCompositeSrc_8888RevNPx0565mmx (pixman_op_t op,
+fbCompositeSrc_8888RevNPx0565mmx (pixman_implementation_t *imp,
+				  pixman_op_t op,
 				  pixman_image_t * pSrc,
 				  pixman_image_t * pMask,
 				  pixman_image_t * pDst,
-				  int16_t      xSrc,
-				  int16_t      ySrc,
-				  int16_t      xMask,
-				  int16_t      yMask,
-				  int16_t      xDst,
-				  int16_t      yDst,
-				  uint16_t     width,
-				  uint16_t     height)
+				  int32_t      xSrc,
+				  int32_t      ySrc,
+				  int32_t      xMask,
+				  int32_t      yMask,
+				  int32_t      xDst,
+				  int32_t      yDst,
+				  int32_t     width,
+				  int32_t     height)
 {
     uint16_t	*dstLine, *dst;
     uint32_t	*srcLine, *src;
@@ -2188,7 +2180,7 @@ fbCompositeSrc_8888RevNPx0565mmx (pixman_op_t op,
 
 		*(__m64 *)dst = vdest;
 	    }
-	    else if (a0 | a1 | a2 | a3)
+	    else if (s0 | s1 | s2 | s3)
 	    {
 		__m64 vdest = *(__m64 *)dst;
 
@@ -2229,18 +2221,19 @@ fbCompositeSrc_8888RevNPx0565mmx (pixman_op_t op,
 /* "8888RevNP" is GdkPixbuf's format: ABGR, non premultiplied */
 
 static void
-fbCompositeSrc_8888RevNPx8888mmx (pixman_op_t op,
+fbCompositeSrc_8888RevNPx8888mmx (pixman_implementation_t *imp,
+				  pixman_op_t op,
 				  pixman_image_t * pSrc,
 				  pixman_image_t * pMask,
 				  pixman_image_t * pDst,
-				  int16_t      xSrc,
-				  int16_t      ySrc,
-				  int16_t      xMask,
-				  int16_t      yMask,
-				  int16_t      xDst,
-				  int16_t      yDst,
-				  uint16_t     width,
-				  uint16_t     height)
+				  int32_t      xSrc,
+				  int32_t      ySrc,
+				  int32_t      xMask,
+				  int32_t      yMask,
+				  int32_t      xDst,
+				  int32_t      yDst,
+				  int32_t     width,
+				  int32_t     height)
 {
     uint32_t	*dstLine, *dst;
     uint32_t	*srcLine, *src;
@@ -2296,7 +2289,7 @@ fbCompositeSrc_8888RevNPx8888mmx (pixman_op_t op,
 
 		*(__m64 *)dst = pack8888 (d0, d1);
 	    }
-	    else if (a0 | a1)
+	    else if (s0 | s1)
 	    {
 		__m64 vdest = *(__m64 *)dst;
 
@@ -2328,18 +2321,19 @@ fbCompositeSrc_8888RevNPx8888mmx (pixman_op_t op,
 }
 
 static void
-fbCompositeSolidMask_nx8888x0565Cmmx (pixman_op_t op,
+fbCompositeSolidMask_nx8888x0565Cmmx (pixman_implementation_t *imp,
+				      pixman_op_t op,
 				      pixman_image_t * pSrc,
 				      pixman_image_t * pMask,
 				      pixman_image_t * pDst,
-				      int16_t      xSrc,
-				      int16_t      ySrc,
-				      int16_t      xMask,
-				      int16_t      yMask,
-				      int16_t      xDst,
-				      int16_t      yDst,
-				      uint16_t     width,
-				      uint16_t     height)
+				      int32_t      xSrc,
+				      int32_t      ySrc,
+				      int32_t      xMask,
+				      int32_t      yMask,
+				      int32_t      xDst,
+				      int32_t      yDst,
+				      int32_t     width,
+				      int32_t     height)
 {
     uint32_t	src, srca;
     uint16_t	*dstLine;
@@ -2435,18 +2429,19 @@ fbCompositeSolidMask_nx8888x0565Cmmx (pixman_op_t op,
 }
 
 static void
-fbCompositeIn_nx8x8mmx (pixman_op_t op,
+fbCompositeIn_nx8x8mmx (pixman_implementation_t *imp,
+			pixman_op_t op,
 			pixman_image_t * pSrc,
 			pixman_image_t * pMask,
 			pixman_image_t * pDst,
-			int16_t      xSrc,
-			int16_t      ySrc,
-			int16_t      xMask,
-			int16_t      yMask,
-			int16_t      xDst,
-			int16_t      yDst,
-			uint16_t     width,
-			uint16_t     height)
+			int32_t      xSrc,
+			int32_t      ySrc,
+			int32_t      xMask,
+			int32_t      yMask,
+			int32_t      xDst,
+			int32_t      yDst,
+			int32_t     width,
+			int32_t     height)
 {
     uint8_t	*dstLine, *dst;
     uint8_t	*maskLine, *mask;
@@ -2519,18 +2514,19 @@ fbCompositeIn_nx8x8mmx (pixman_op_t op,
 }
 
 static void
-fbCompositeIn_8x8mmx (pixman_op_t op,
+fbCompositeIn_8x8mmx (pixman_implementation_t *imp,
+		      pixman_op_t op,
 		      pixman_image_t * pSrc,
 		      pixman_image_t * pMask,
 		      pixman_image_t * pDst,
-		      int16_t      xSrc,
-		      int16_t      ySrc,
-		      int16_t      xMask,
-		      int16_t      yMask,
-		      int16_t      xDst,
-		      int16_t      yDst,
-		      uint16_t     width,
-		      uint16_t     height)
+		      int32_t      xSrc,
+		      int32_t      ySrc,
+		      int32_t      xMask,
+		      int32_t      yMask,
+		      int32_t      xDst,
+		      int32_t      yDst,
+		      int32_t     width,
+		      int32_t     height)
 {
     uint8_t	*dstLine, *dst;
     uint8_t	*srcLine, *src;
@@ -2583,18 +2579,19 @@ fbCompositeIn_8x8mmx (pixman_op_t op,
 }
 
 static void
-fbCompositeSrcAdd_8888x8x8mmx (pixman_op_t op,
+fbCompositeSrcAdd_8888x8x8mmx (pixman_implementation_t *imp,
+			       pixman_op_t op,
 			       pixman_image_t * pSrc,
 			       pixman_image_t * pMask,
 			       pixman_image_t * pDst,
-			       int16_t      xSrc,
-			       int16_t      ySrc,
-			       int16_t      xMask,
-			       int16_t      yMask,
-			       int16_t      xDst,
-			       int16_t      yDst,
-			       uint16_t     width,
-			       uint16_t     height)
+			       int32_t      xSrc,
+			       int32_t      ySrc,
+			       int32_t      xMask,
+			       int32_t      yMask,
+			       int32_t      xDst,
+			       int32_t      yDst,
+			       int32_t     width,
+			       int32_t     height)
 {
     uint8_t	*dstLine, *dst;
     uint8_t	*maskLine, *mask;
@@ -2661,18 +2658,19 @@ fbCompositeSrcAdd_8888x8x8mmx (pixman_op_t op,
 }
 
 static void
-fbCompositeSrcAdd_8000x8000mmx (pixman_op_t op,
+fbCompositeSrcAdd_8000x8000mmx (pixman_implementation_t *imp,
+				pixman_op_t op,
 				pixman_image_t * pSrc,
 				pixman_image_t * pMask,
 				pixman_image_t * pDst,
-				int16_t      xSrc,
-				int16_t      ySrc,
-				int16_t      xMask,
-				int16_t      yMask,
-				int16_t      xDst,
-				int16_t      yDst,
-				uint16_t     width,
-				uint16_t     height)
+				int32_t      xSrc,
+				int32_t      ySrc,
+				int32_t      xMask,
+				int32_t      yMask,
+				int32_t      xDst,
+				int32_t      yDst,
+				int32_t     width,
+				int32_t     height)
 {
     uint8_t	*dstLine, *dst;
     uint8_t	*srcLine, *src;
@@ -2733,18 +2731,19 @@ fbCompositeSrcAdd_8000x8000mmx (pixman_op_t op,
 }
 
 static void
-fbCompositeSrcAdd_8888x8888mmx (pixman_op_t 	op,
+fbCompositeSrcAdd_8888x8888mmx (pixman_implementation_t *imp,
+				pixman_op_t 	op,
 				pixman_image_t *	pSrc,
 				pixman_image_t *	pMask,
 				pixman_image_t *	 pDst,
-				int16_t		 xSrc,
-				int16_t      ySrc,
-				int16_t      xMask,
-				int16_t      yMask,
-				int16_t      xDst,
-				int16_t      yDst,
-				uint16_t     width,
-				uint16_t     height)
+				int32_t		 xSrc,
+				int32_t      ySrc,
+				int32_t      xMask,
+				int32_t      yMask,
+				int32_t      xDst,
+				int32_t      yDst,
+				int32_t     width,
+				int32_t     height)
 {
     __m64 dst64;
     uint32_t	*dstLine, *dst;
@@ -2794,7 +2793,7 @@ fbCompositeSrcAdd_8888x8888mmx (pixman_op_t 	op,
     _mm_empty();
 }
 
-pixman_bool_t
+static pixman_bool_t
 pixman_blt_mmx (uint32_t *src_bits,
 		uint32_t *dst_bits,
 		int src_stride,
@@ -2930,19 +2929,20 @@ pixman_blt_mmx (uint32_t *src_bits,
     return TRUE;
 }
 
-void
-fbCompositeCopyAreammx (pixman_op_t       op,
+static void
+fbCompositeCopyAreammx (pixman_implementation_t *imp,
+			pixman_op_t       op,
 			pixman_image_t *	pSrc,
 			pixman_image_t *	pMask,
 			pixman_image_t *	pDst,
-			int16_t		xSrc,
-			int16_t		ySrc,
-			int16_t		xMask,
-			int16_t		yMask,
-			int16_t		xDst,
-			int16_t		yDst,
-			uint16_t		width,
-			uint16_t		height)
+			int32_t		xSrc,
+			int32_t		ySrc,
+			int32_t		xMask,
+			int32_t		yMask,
+			int32_t		xDst,
+			int32_t		yDst,
+			int32_t		width,
+			int32_t		height)
 {
     pixman_blt_mmx (pSrc->bits.bits,
 		    pDst->bits.bits,
@@ -2954,18 +2954,19 @@ fbCompositeCopyAreammx (pixman_op_t       op,
 }
 
 static void
-fbCompositeOver_x888x8x8888mmx (pixman_op_t      op,
+fbCompositeOver_x888x8x8888mmx (pixman_implementation_t *imp,
+				pixman_op_t      op,
 				pixman_image_t * pSrc,
 				pixman_image_t * pMask,
 				pixman_image_t * pDst,
-				int16_t      xSrc,
-				int16_t      ySrc,
-				int16_t      xMask,
-				int16_t      yMask,
-				int16_t      xDst,
-				int16_t      yDst,
-				uint16_t     width,
-				uint16_t     height)
+				int32_t      xSrc,
+				int32_t      ySrc,
+				int32_t      xMask,
+				int32_t      yMask,
+				int32_t      xDst,
+				int32_t      yDst,
+				int32_t     width,
+				int32_t     height)
 {
     uint32_t	*src, *srcLine;
     uint32_t    *dst, *dstLine;
@@ -3017,7 +3018,7 @@ fbCompositeOver_x888x8x8888mmx (pixman_op_t      op,
     _mm_empty();
 }
 
-static const FastPathInfo mmx_fast_path_array[] =
+static const FastPathInfo mmx_fast_paths[] =
 {
     { PIXMAN_OP_OVER, PIXMAN_solid,    PIXMAN_a8,       PIXMAN_r5g6b5,   fbCompositeSolidMask_nx8x0565mmx,     0 },
     { PIXMAN_OP_OVER, PIXMAN_solid,    PIXMAN_a8,       PIXMAN_b5g6r5,   fbCompositeSolidMask_nx8x0565mmx,     0 },
@@ -3063,6 +3064,7 @@ static const FastPathInfo mmx_fast_path_array[] =
     { PIXMAN_OP_OVER, PIXMAN_solid,    PIXMAN_null,     PIXMAN_r5g6b5,   fbCompositeSolid_nx0565mmx,	   0 },
     { PIXMAN_OP_OVER, PIXMAN_x8r8g8b8, PIXMAN_null,     PIXMAN_x8r8g8b8, fbCompositeCopyAreammx,	   0 },
     { PIXMAN_OP_OVER, PIXMAN_x8b8g8r8, PIXMAN_null,     PIXMAN_x8b8g8r8, fbCompositeCopyAreammx,	   0 },
+
     { PIXMAN_OP_OVER, PIXMAN_a8r8g8b8, PIXMAN_null,     PIXMAN_a8r8g8b8, fbCompositeSrc_8888x8888mmx,	   0 },
     { PIXMAN_OP_OVER, PIXMAN_a8r8g8b8, PIXMAN_null,	PIXMAN_x8r8g8b8, fbCompositeSrc_8888x8888mmx,	   0 },
     { PIXMAN_OP_OVER, PIXMAN_a8r8g8b8, PIXMAN_null,	PIXMAN_r5g6b5,	 fbCompositeSrc_8888x0565mmx,	   0 },
@@ -3093,6 +3095,117 @@ static const FastPathInfo mmx_fast_path_array[] =
 
     { PIXMAN_OP_NONE },
 };
-const FastPathInfo *const mmx_fast_paths = mmx_fast_path_array;
+
+static void
+mmx_composite (pixman_implementation_t *imp,
+	       pixman_op_t     op,
+	       pixman_image_t *src,
+	       pixman_image_t *mask,
+	       pixman_image_t *dest,
+	       int32_t         src_x,
+	       int32_t         src_y,
+	       int32_t         mask_x,
+	       int32_t         mask_y,
+	       int32_t         dest_x,
+	       int32_t         dest_y,
+	       int32_t         width,
+	       int32_t         height)
+{
+    if (_pixman_run_fast_path (mmx_fast_paths, imp,
+			       op, src, mask, dest,
+			       src_x, src_y,
+			       mask_x, mask_y,
+			       dest_x, dest_y,
+			       width, height))
+	return;
+
+    _pixman_implementation_composite (imp->delegate,
+				      op, src, mask, dest, src_x, src_y,
+				      mask_x, mask_y, dest_x, dest_y,
+				      width, height);
+}
+
+static pixman_bool_t
+mmx_blt (pixman_implementation_t *imp,
+	 uint32_t *src_bits,
+	 uint32_t *dst_bits,
+	 int src_stride,
+	 int dst_stride,
+	 int src_bpp,
+	 int dst_bpp,
+	 int src_x, int src_y,
+	 int dst_x, int dst_y,
+	 int width, int height)
+{
+    if (!pixman_blt_mmx (
+	    src_bits, dst_bits, src_stride, dst_stride, src_bpp, dst_bpp,
+	    src_x, src_y, dst_x, dst_y, width, height))
+
+    {
+	return _pixman_implementation_blt (
+	    imp->delegate,
+	    src_bits, dst_bits, src_stride, dst_stride, src_bpp, dst_bpp,
+	    src_x, src_y, dst_x, dst_y, width, height);
+    }
+
+    return TRUE;
+}
+
+static pixman_bool_t
+mmx_fill (pixman_implementation_t *imp,
+	  uint32_t *bits,
+	  int stride,
+	  int bpp,
+	  int x,
+	  int y,
+	  int width,
+	  int height,
+	  uint32_t xor)
+{
+    if (!pixman_fill_mmx (bits, stride, bpp, x, y, width, height, xor))
+    {
+	return _pixman_implementation_fill (
+	    imp->delegate, bits, stride, bpp, x, y, width, height, xor);
+    }
+
+    return TRUE;
+}
+
+pixman_implementation_t *
+_pixman_implementation_create_mmx (pixman_implementation_t *toplevel)
+{
+    pixman_implementation_t *general = _pixman_implementation_create_fast_path (NULL);
+    pixman_implementation_t *imp = _pixman_implementation_create (toplevel, general);
+
+    imp->combine_32[PIXMAN_OP_OVER] = mmxCombineOverU;
+    imp->combine_32[PIXMAN_OP_OVER_REVERSE] = mmxCombineOverReverseU;
+    imp->combine_32[PIXMAN_OP_IN] = mmxCombineInU;
+    imp->combine_32[PIXMAN_OP_IN_REVERSE] = mmxCombineInReverseU;
+    imp->combine_32[PIXMAN_OP_OUT] = mmxCombineOutU;
+    imp->combine_32[PIXMAN_OP_OUT_REVERSE] = mmxCombineOutReverseU;
+    imp->combine_32[PIXMAN_OP_ATOP] = mmxCombineAtopU;
+    imp->combine_32[PIXMAN_OP_ATOP_REVERSE] = mmxCombineAtopReverseU;
+    imp->combine_32[PIXMAN_OP_XOR] = mmxCombineXorU; 
+    imp->combine_32[PIXMAN_OP_ADD] = mmxCombineAddU;
+    imp->combine_32[PIXMAN_OP_SATURATE] = mmxCombineSaturateU;
+    
+    imp->combine_32_ca[PIXMAN_OP_SRC] = mmxCombineSrcC;
+    imp->combine_32_ca[PIXMAN_OP_OVER] = mmxCombineOverC;
+    imp->combine_32_ca[PIXMAN_OP_OVER_REVERSE] = mmxCombineOverReverseC;
+    imp->combine_32_ca[PIXMAN_OP_IN] = mmxCombineInC;
+    imp->combine_32_ca[PIXMAN_OP_IN_REVERSE] = mmxCombineInReverseC;
+    imp->combine_32_ca[PIXMAN_OP_OUT] = mmxCombineOutC;
+    imp->combine_32_ca[PIXMAN_OP_OUT_REVERSE] = mmxCombineOutReverseC;
+    imp->combine_32_ca[PIXMAN_OP_ATOP] = mmxCombineAtopC;
+    imp->combine_32_ca[PIXMAN_OP_ATOP_REVERSE] = mmxCombineAtopReverseC;
+    imp->combine_32_ca[PIXMAN_OP_XOR] = mmxCombineXorC;
+    imp->combine_32_ca[PIXMAN_OP_ADD] = mmxCombineAddC;
+
+    imp->composite = mmx_composite;
+    imp->blt = mmx_blt;
+    imp->fill = mmx_fill;
+    
+    return imp;
+}
 
 #endif /* USE_MMX */
